{% extends "account/base.html" %}

{% load i18n %}
{% load bootstrap_tags %}

{% block head_title %}{% trans "Change Password" %}{% endblock %}

{% block content %}
    <h1>{% if token_fail %}{% trans "Bad Token" %}{% else %}{% trans "Change Password" %}{% endif %}</h1>

    {% if token_fail %}
        {% url 'account_reset_password' as passwd_reset_url %}
        <p>{% blocktrans %}The password reset link was invalid, possibly because it has already been used.  Please request a <a href="{{ passwd_reset_url }}">new password reset</a>.{% endblocktrans %}</p>
    {% else %}
        {% if form %}
            <form method="POST" action="." class="form-horizontal">
                {% bootstrap_form_horizontal form 2 4 %}
                <div class="form-group">
                  <div class="offset-md-2 col-md-10">
                    <input class="btn btn-primary" type="submit" name="action" value="{% trans 'change password' %}"/>
                  </div>
                </div>
            </form>
        {% else %}
            <p>{% trans 'Your password is now changed.' %}</p>
        {% endif %}
    {% endif %}
{% endblock %}
