{% extends "socialaccount/base.html" %}

{% load i18n %}
{% load bootstrap_tags %}

{% block head_title %}{% trans "Signup" %}{% endblock %}

{% block content %}
    <h1>{% trans "Sign Up" %}</h1>

<p>{% blocktrans with provider_name=account.get_provider.name site_name=site.name %}You are about to use your {{provider_name}} account to login to
{{site_name}}. As a final step, please complete the following form:{% endblocktrans %}</p>

<form class="signup form-horizontal" id="signup_form" method="post" action="{% url 'socialaccount_signup' %}">
  {% bootstrap_form_horizontal form 2 4 %}
  {% if redirect_field_value %}
  <input type="hidden" name="{{ redirect_field_name }}" value="{{ redirect_field_value }}" />
  {% endif %}
  <div class="form-group">
    <div class="offset-md-2 col-md-4">
      <button class="btn btn-primary" type="submit">{% trans "Sign Up" %} &raquo;</button>
    </div>
  </div>
</form>

{% endblock %}
