# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 by the Free Software Foundation, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

from allauth.account.models import EmailAddress


def get_django_user(member):
    """Get the Django user from the Member.

    :param member: The Mailman member resource.
    :returns: A Django User instance, if one exists.
    """
    try:
        email_addr = EmailAddress.objects.get(email=member.address.email)
    except EmailAddress.DoesNotExist:
        return None
    return email_addr.user
